package com.logicbig.example;

import javax.persistence.*;
import java.util.Map;

public class ExampleMain {

    public static void main(String[] args) {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("example-unit");
        try {
            persistEntity(emf);
            findItemQty(emf);
            findOrderMapEntry(emf);
            findCustomerByItemName(emf);
        } finally {
            emf.close();
        }
    }

    private static void persistEntity(EntityManagerFactory emf) {
        System.out.println("-- Persisting entities --");
        EntityManager em = emf.createEntityManager();

        Customer c1 = new Customer();
        c1.setName("Lindsey Craft");
        c1.addOrder("online", "XYZ Blender", 2);
        c1.addOrder("store", "ZZZ Beer Glass", 4);
        System.out.println(c1);

        Customer c2 = new Customer();
        c2.setName("Morgan Philips");
        c2.addOrder("online", "AA Glass Cleaner", 3);
        System.out.println(c2);

        em.getTransaction().begin();
        em.persist(c1);
        em.persist(c2);
        em.getTransaction().commit();
        em.close();
    }

    private static void findItemQty(EntityManagerFactory emf) {
        System.out.println("-- Finding items and keys --");
        EntityManager em = emf.createEntityManager();
        TypedQuery<Tuple> query =
                em.createQuery("SELECT c.name, KEY(map), VALUE(map).item, VALUE(map).qty "
                        + "FROM Customer c JOIN c.orderMap map", Tuple.class);
        query.getResultList().forEach(
                t -> System.out.printf("Customer: %s, Order-Type: %s, Order-item: %s, Order-qty: %s%n",
                        t.get(0), t.get(1), t.get(2), t.get(3)));
    }

    private static void findOrderMapEntry(EntityManagerFactory emf) {
        System.out.println("-- Finding item/keys entries  --");
        EntityManager em = emf.createEntityManager();
        TypedQuery<Map.Entry> query =
                em.createQuery("SELECT ENTRY(c.orderMap) FROM Customer c", Map.Entry.class);
        query.getResultList().forEach(entry -> System.out.printf("Order-Type: %s, Order: %s%n",
                entry.getKey(), entry.getValue()));
    }

    private static void findCustomerByItemName(EntityManagerFactory emf) {
        System.out.println("-- Finding  customers where item name like %Beer% --");
        EntityManager em = emf.createEntityManager();
        TypedQuery<Customer> query =
                em.createQuery("SELECT c FROM Customer c"
                        + " WHERE VALUE(c.orderMap).item LIKE '%Beer%'", Customer.class);
        query.getResultList().forEach(System.out::println);
    }
}